@@ -20,14 +20,13 @@
 
 #if !defined(GPR_LINUX) && !defined(GPR_WINDOWS)
 
-#include <grpc/support/log.h>
+#include "absl/log/log.h"
 
 #include "src/core/lib/gprpp/crash.h"
 #include "src/core/lib/security/credentials/alts/check_gcp_environment.h"
 
 bool grpc_alts_is_running_on_gcp() {
-  gpr_log(GPR_INFO,
-          "ALTS: Platforms other than Linux and Windows are not supported");
+  VLOG(2) << "ALTS: Platforms other than Linux and Windows are not supported";
   return false;
 }
 
